import { gql, useQuery } from "@apollo/client";
import { Link, useRouter } from "@curi/react-dom";
import {
  List,
  ListItem,
  Typography,
  ListItemText,
  Menu,
  MenuItem,
  PopoverPosition,
} from "@mui/material";
import { FC, useMemo, useState, MouseEvent } from "react";
import { Pipeline, Project } from "../generated/graphql";
import { CallMerge } from "@mui/icons-material";
import { Divider } from "@material-ui/core";

interface Props {
  projectId: string;
}

const PIPELINES = gql`
  query Pipelines($projectId: String!) {
    pipelines(projectId: $projectId) {
      id
      name
      branch
    }
    project(id: $projectId) {
      id
    }
  }
`;

export const PipelineList: FC<Props> = ({ projectId }) => {
  const { data } = useQuery<
    { pipelines: Pipeline[]; project: Project },
    { projectId: string }
  >(PIPELINES, {
    variables: { projectId },
  });
  const pipelines = useMemo(() => {
    return data?.pipelines?.map((pipeline) => ({
      ...pipeline,
      project: data?.project,
    }));
  }, [data]);

  const [contextMenu, setContextMenu] = useState<[PopoverPosition, Pipeline]>();
  const { navigate, url } = useRouter();

  const handleContextMenu = (event: MouseEvent, pipeline: Pipeline | false) => {
    event.preventDefault();
    if (pipeline) {
      setContextMenu([
        { top: event.clientY - 4, left: event.clientX - 2 },
        pipeline,
      ]);
    } else {
      setContextMenu(undefined);
    }
  };

  const handleClose = () => {
    setContextMenu(undefined);
  };

  const modify = () => {
    navigate({
      url: url({
        name: "edit-pipeline",
        params: {
          pipelineId: contextMenu![1].id,
          projectId: contextMenu![1].project.id,
        },
      }),
    });
  };
  const modifyRuntimeConfiguration = () => {
    navigate({
      url: url({
        name: "edit-runtime-configuration",
        params: {
          pipelineId: contextMenu![1].id,
          projectId: contextMenu![1].project.id,
        },
      }),
    });
  };

  return (
    <>
      <List>
        {pipelines?.map((pipeline) => (
          <Link
            name="pipeline-commits"
            params={{ pipelineId: pipeline.id, projectId: projectId }}
            key={pipeline.id}
            onContextMenu={(ev) => handleContextMenu(ev, pipeline)}
          >
            <Item pipeline={pipeline} />
          </Link>
        ))}
      </List>
      <Menu
        keepMounted
        open={!!contextMenu}
        onClose={handleClose}
        onContextMenu={(ev) => handleContextMenu(ev, false)}
        anchorReference="anchorPosition"
        anchorPosition={contextMenu?.[0]}
      >
        <MenuItem
          onClick={() => {
            modifyRuntimeConfiguration();
            handleClose();
          }}
        >
          Runtime Config
        </MenuItem>
        <MenuItem
          onClick={() => {
            modify();
            handleClose();
          }}
        >
          Pipeline Config
        </MenuItem>
        <Divider />
        <MenuItem sx={{ color: "error.main" }} onClick={handleClose}>
          Delete
        </MenuItem>
      </Menu>
    </>
  );
};

const Item = ({ pipeline }: { pipeline: Pipeline }) => {
  return (
    <ListItem button>
      <ListItemText
        primary={pipeline.name}
        secondary={
          <Typography component="span" variant="body2" color="textSecondary">
            <CallMerge fontSize="small" />
            {pipeline.branch}
          </Typography>
        }
      />
    </ListItem>
  );
};
